
#include <FastLED.h>
#include <Adafruit_NeoPixel.h>
#include <Wire.h>
#include "pitches.h"
#include "SparkFun_CAP1203.h"
#ifdef __AVR__
  #include <avr/power.h>
#endif

#define COLOR_ORDER GRB
#define VOLTS 5
#define MAX_AMPS 500
#define CHIPSET WS2812B

#define ROTARY_PINA 6
#define ROTARY_PINB 5

#define BUTT_RED_PIN 12
#define BUTT_BLUE_PIN 10
#define BUTT_GREEN_PIN 11
#define BUTT_INPUT_PIN 9

#define SPEAKER_PIN 3

#define LED_PIN 4
#define WHEEL_PIN 2
#define NUM_WHEELLEDS 27
#define NUM_LEDS 4

CRGB leds[NUM_LEDS];
CRGB inputColors[NUM_LEDS];
Adafruit_NeoPixel wheel = Adafruit_NeoPixel(NUM_WHEELLEDS, WHEEL_PIN, NEO_GRB + NEO_KHZ800);

CRGB state[2];

// for rotary encoder and button
int lastStateCLK;
int currStateCLK;
int currDirection; // Determine the color
int colorOutPINS[3] = {BUTT_RED_PIN, BUTT_GREEN_PIN, BUTT_BLUE_PIN};
CRGB colorInRotary[3] = {CRGB(255, 0, 0), CRGB(0, 255, 0), CRGB(0, 0, 255)};


// color for wheel
CRGB colorInWheels[15] = {
  CRGB(255, 0, 0), CRGB(191, 63, 0), CRGB(127, 127, 0), CRGB(63, 191, 0),
  CRGB(0, 255, 0), CRGB(0, 191, 63), CRGB(0, 127, 127), CRGB(0, 63, 191),
  CRGB(0, 0, 255), CRGB(63, 0, 191), CRGB(127, 0, 127), CRGB(191, 0, 63),
  CRGB(63, 63, 127), CRGB(127, 63, 63), CRGB(63, 127, 63)  
};

// private state
int currentLed = 0;
int currentCreatedColor = 0;
int STATE = 0;
float timer = 0;
int MODE = 0;

// speaker
int valuesToNote[] = {
  NOTE_C4, NOTE_E4, NOTE_G4
};
int noteDuration = 1000 / 4;
int newNote = NOTE_C7;
int oldNote = NOTE_B2;
bool SPEAKER_TUNRON = true;

// for audio controller sensor
CAP1203 sensor;
int isTouchingPad = 0;


void setup() {

  pinMode(ROTARY_PINA, INPUT);
  pinMode(ROTARY_PINB, INPUT);
  lastStateCLK = digitalRead(ROTARY_PINA);

  pinMode(BUTT_INPUT_PIN, INPUT);
  pinMode(BUTT_RED_PIN, OUTPUT);
  digitalWrite(BUTT_RED_PIN, LOW); 
  pinMode(BUTT_BLUE_PIN, OUTPUT);
  digitalWrite(BUTT_BLUE_PIN, HIGH); 
  pinMode(BUTT_GREEN_PIN, OUTPUT);
  digitalWrite(BUTT_GREEN_PIN, HIGH);
  currDirection = 0; 
  
  FastLED.addLeds<CHIPSET, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS);
  FastLED.setMaxPowerInVoltsAndMilliamps(VOLTS, MAX_AMPS);
  FastLED.setBrightness(50);
  FastLED.clear();
  FastLED.show();

  wheel.begin();
  wheel.setBrightness(50);
  wheel.show();

  // Setup sensor
  while (sensor.begin() == false) {
    sensor.begin();
  }
  
  sensor.setPowerButtonPad(PAD_MIDDLE);
  sensor.setSensitivity(SENSITIVITY_2X);
  sensor.setPowerButtonTime(PWR_TIME_280_MS);
  sensor.setPowerButtonEnabled();

  //lightShow();

  Serial.begin(9600);
}

void loop() {
  
  setCurrColorByRotary();
  detectingPad();

  if (STATE == 0 && isPressing()) {
     STATE = 1;
     leds[currentLed] = colorInRotary[currDirection];
     FastLED.show();
     inputColors[currentLed] = colorInRotary[currDirection];
     delay(250);
     STATE = 0;
     currentLed++;
  } 
//  else if (STATE == 1 && !isPressing()) {
//     STATE = 0;
//     currentLed++;
//     Serial.print("CurrLed");
//     Serial.println(currentLed);
//  }

  if (currentLed == NUM_LEDS) {
    STATE = 2;
    combineOneByOne();
  }
  
  if (currentCreatedColor == 12) {
      delay(700);
      lightShow();
      reset();
  }
}


void setCurrColorByRotary() {
  currStateCLK = digitalRead(ROTARY_PINA);
  if (currStateCLK != lastStateCLK  && currStateCLK == 1){
    digitalWrite(colorOutPINS[currDirection], HIGH); 
    
    if (digitalRead(ROTARY_PINB) != currStateCLK) {
      currDirection --;
      if (currDirection < 0 ) {
        currDirection += sizeof(colorInRotary) / sizeof(CRGB);
      }
    } else {
      currDirection ++;
      currDirection %= sizeof(colorInRotary) / sizeof(CRGB);
    }
    
    digitalWrite(colorOutPINS[currDirection], LOW); 
  }
  lastStateCLK = currStateCLK;
}

void detectingPad() {
  if (isTouchingPad == 0 && sensor.isPowerButtonTouched()) {
    isTouchingPad = 1;
  } else if (isTouchingPad != 0 && sensor.isPowerButtonTouched()) {
    isTouchingPad += 1;
    if (isTouchingPad == 3000) changeMode();
  } else if (isTouchingPad != 0 && !sensor.isPowerButtonTouched()) {
    if (isTouchingPad < 3000) SPEAKER_TUNRON = !SPEAKER_TUNRON;
    isTouchingPad = 0;
  }
}

void changeMode() {
  if (MODE == 1) MODE = 0;
  else if (MODE == 0) MODE = 1;
  reset();
}

void setCenterLight(CRGB color) {
  uint32_t wheelColor = wheel.Color(color.red, color.green, color.blue);
  for (int i = 24; i < wheel.numPixels(); ++i) {
    wheel.setPixelColor(i, wheelColor);
  }
  wheel.show();
}

bool isPressing() {
//  Serial.print("isPressing");
//  Serial.println(digitalRead(BUTT_INPUT_PIN));
  return digitalRead(BUTT_INPUT_PIN) == HIGH;
}

void combineOneByOne() {
  delay(100);
  float r = inputColors[0].red; 
  float g = inputColors[0].green; 
  float b = inputColors[0].blue;
  for (int i = 1; i < NUM_LEDS; i++) {
    r += inputColors[i].red;
    g += inputColors[i].green;
    b += inputColors[i].blue;
    leds[i - 1] = CRGB(0, 0, 0);
    leds[i] = CRGB(r / (i + 1), g / (i + 1), b / (i + 1));
    FastLED.show();
    playSound(valuesToNote[i - 1], noteDuration);
    delay(500);
  }
  CRGB combineColor = leds[NUM_LEDS - 1];
//  Serial.println(combineColor.red);
//  Serial.println(combineColor.green);
//  Serial.println(combineColor.blue);
//  Serial.println("-----");
  
  leds[NUM_LEDS - 1] = CRGB(0, 0, 0);
 
  FastLED.show();
  STATE = 0;
  currentLed = 0;

  if (MODE == 0) lightWheel_Quiz(combineColor);
  else if (MODE == 1) lightWheel_Free(combineColor);
}


void lightWheel_Free(CRGB color) {
  int currIndex = currentCreatedColor * 2;
  uint32_t wheelColor = wheel.Color(color.red, color.green, color.blue);
  wheel.setPixelColor(currIndex, wheelColor);
  wheel.setPixelColor(currIndex + 1, wheelColor);
  wheel.show();
  currentCreatedColor++;
  playSound(newNote, noteDuration);
}

void lightWheel_Quiz(CRGB color) {
   int index = getColorIndex(color);
   if (index < 0) return;
   if (index < 12) index *= 2;
   else index = 24 + (index - 12);
   
   uint32_t wheelColor = index < 24 ? 
      wheel.Color(color.red, color.green, color.blue) :
      wheel.Color(color.green, color.red, color.blue) ;
   uint32_t dark = wheel.Color(0, 0, 0);
   
   if (wheel.getPixelColor(index) == dark) {
      if (index < 24) currentCreatedColor++;
      playSound(newNote, noteDuration);
   } else {
      playSound(oldNote, noteDuration);
      for (int i = 0; i < 6; i++) {
        uint32_t tempColor = i % 2 == 0 ? dark : wheelColor;
        wheel.setPixelColor(index, tempColor);
        if (index < 24) wheel.setPixelColor(index + 1, tempColor);
        wheel.show();
        delay(100);
      }
   }

   wheel.setPixelColor(index, wheelColor);
   if (index < 24) wheel.setPixelColor(index + 1, wheelColor);
   wheel.show();
}

void playSound(int note, int duration) {
  if (SPEAKER_TUNRON) tone(SPEAKER_PIN, note, duration);
}


void lightShow() {
  delay(100);
  if (MODE == 0) rainbowCycle(10);
  else if (MODE == 1) rainbowCycle_free(20);

}

void reset() {
  wheel.clear();
  wheel.setBrightness(50);
  if (MODE == 1) {
    delay(600);
    setCenterLight(CRGB(255, 255, 255));
  }
  
  wheel.show();
  FastLED.clear();
  FastLED.show();
  STATE = 0;
  currentLed = 0;
  currentCreatedColor = 0;
}


// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint32_t i, j;
  uint32_t maxVal = 256*4;
  for(j=0; j<maxVal; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< wheel.numPixels(); i++) {
      wheel.setPixelColor(i, Wheel(((i * 256 / wheel.numPixels()) + j) & 255));
    }
    wheel.setBrightness(90 * (maxVal - j) / maxVal);
    wheel.show();
    delay(wait);
  }
}

void rainbowCycle_free(uint8_t wait) {
  uint16_t i, j;
  uint16_t maxVal = 300;
  uint32_t colors[24];
  
  for(i=0; i<24; i++) {
    colors[i] = wheel.getPixelColor(i);
  }
 
  for(j=0; j<maxVal; j++) { // 5 cycles of all colors on wheel
    for(i=0; i<24; i++) {
      wheel.setPixelColor(i, colors[(i + j) % 24]);
    }
    wheel.setBrightness(50 * (maxVal - j) / maxVal);
    wheel.show();
    delay(wait);
  }
}


//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) {
  for (int j=0; j < 256; j++) {     // cycle all 256 colors in the wheel
    for (int q=0; q < 2; q++) {
      for (uint16_t i=0; i < wheel.numPixels(); i=i+2) {
        wheel.setPixelColor(i+q, Wheel( (i+j) % 255));    //turn every third pixel on
      }
      wheel.show();

      delay(wait);

      for (uint16_t i=0; i < wheel.numPixels(); i=i+2) {
        wheel.setPixelColor(i+q, 0);        //turn every third pixel off
      }
    }
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return wheel.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return wheel.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return wheel.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}


int getColorIndex(CRGB color) {
  int index = -1;
  for (int i = 0; i < sizeof(colorInWheels) / sizeof(CRGB); ++i) {
    if (colorInWheels[i] != color) continue;
    index = i; break;
  }
  return index;
}
